#!/usr/bin/perl

###############################################################################
### parse_wordlist.pl
### this script is meant to parse a text file containing document names and
### lists of words contained in each document.  output consists of three files:
### a term vector, a document vector, and a term X document binary matrix.
###############################################################################

# note: matrix output needs to be revamped

$INPUT_FILE = "./data.file";
$DESTINATION_DIRECTORY = "./";
$TERM_FILE = $DESTINATION_DIRECTORY . "terms.txt";
$DOCUMENT_FILE = $DESTINATION_DIRECTORY . "documents.txt";
$MATRIX_FILE = $DESTINATION_DIRECTORY . "matrix.txt";

$DOCUMENT_NAME_DELIMITER = "\n\n";
$WORD_DELIMITER = "\n";

        #######################################################################
	### check for input filename from command line otherwise use default
        #######################################################################

	if(@ARGV == 1)  ## input file has been specified from command line
	{
		$input_file = $ARGV[0];
	}
	else
	{
		$input_file = $INPUT_FILE;
	}

        #######################################################################
	### if an input filename is defined, parse it
        #######################################################################

	if($input_file ne "")
	{
		$input = `cat $input_file`;

		@input_ary = split /$DOCUMENT_NAME_DELIMITER/, $input;

		###############################################################
		### even numbered indices contain document names
		### odd numbered indices contain word lists
		###############################################################

		### first, get an array of all possible words

		for($i = 0; $i < @input_ary; $i++)
		{
			if(!($i % 2))
			{
				@tmp_words = split /$WORD_DELIMITER/, $input_ary[$i+1];
				push @words, @tmp_words;
				push @document_names, $input_ary[$i];
				$docs{$input_ary[$i]} = [ @tmp_words ];

				# print "$input_ary[$i] \n\n $input_ary[$i+1] \n\n";
				print "GATHERING: " . $input_ary[$i] . "\n";
			}
		}

		### sort the array alphabetically, ignore case

		@sorted_words = sort{lc($a) cmp lc($b)} @words;

		## 1114 ## %matrix = ();
		#$matrix = "";
		$non_zero_values = 0;

		$row = 0;
		$previous_word = "";

		open MATRIX_OUT, ">$MATRIX_FILE";

		# check each word
		foreach $word (@sorted_words)
		{
			if($word eq $previous_word)
			{
				$previous_word = $word;
				next;
			}
			else
			{
				$row++;
				push @final_wordlist, $word;

				$col = 0;

				# check each document
				foreach $doc (@document_names)
				{
					$col++;
					$value = 0;

					# foreach $docword (@$docs{$doc})
					# check each word in this document
					# to see if it matches current word
					for $i (0 .. $#{ $docs{$doc} })
					{
						$docword = $docs{$doc}[$i];

						# if($docword =~ /$word/si)
						# 02-01-2002 decided to use
						# exact match instead of substr
						# match

						if($docword eq $word)
						{
							$value++;

							print "ADDING: $word | $docword | $value | $doc\n\n";
						}
						elsif($value > 0)
						{
							last;
						}
					}

					if($value > 0)
					{
						$non_zero_values++;

						# $matrix .= "$row $col  $value\n";
						print MATRIX_OUT "$row $col  $value\n";
						print "$word | $row $col $value | $docword\n";
						# $ch = <STDIN>;
					}	

					## 1114 ## $matrix{$word} .= $value;
				}

				$previous_word = $word;
			}

			# print $word . "\n";
		}
	}
	else
	{
		die("No input file was specified");
	}

	#######################################################################
	### output document vector
	#######################################################################

	open DOCUMENT_OUT, ">$DOCUMENT_FILE";

	foreach $document (@document_names)
	{
		print DOCUMENT_OUT "$document\n";
	}

	close(DOCUMENT_OUT);

	#######################################################################
	### output word vector and matrix
	#######################################################################

	$num_rows = $#final_wordlist + 1;
	$num_cols = $#document_names + 1;

	open TERM_OUT, ">$TERM_FILE";

	foreach $word (@final_wordlist)  ### keys(%matrix))
	{ 
		print TERM_OUT "$word\n";
		## 1114 ## print MATRIX_OUT "$matrix{$word}\n";
	}

	## 1114 ## close(MATRIX_OUT);
	close(TERM_OUT);

	# print MATRIX_OUT "% term by document matrix for search engine\n";
	print "$num_rows $num_cols $non_zero_values\n";
	# print MATRIX_OUT "$matrix";
	close(MATRIX_OUT);

