#include <stdio.h>
#include <string.h>
#include <math.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
#include <vector>
#include <string>
#include <algorithm>

#define THRESHOLD 0.132
#define MATCH_LIMIT 15

class match
{
	public:
		int doc_index;
		double cosine;
		
		match() {}
		match(int i, double c)
		{
			doc_index = i;
			cosine = c;
		}
		bool operator<(const match& b) const
		{
			return cosine < b.cosine;
		}
};

int
main (int argc, char* argv[])
{
	if(argc < 2)
	{
		printf("%s\n", "no matching records");
		return 1;
	}

	int i = 0;
	int num_terms = argc - 1;
	int num_matches = 0;
	double current = 0;
	char buffer[80] = "";
	char a[20] = "";
	char b[20] = "";
	char c[20] = "";
	gsl_matrix* termByDoc;
	gsl_vector* searchTerm;
	char* search[num_terms];
	vector<string> term;
	vector<string> doc;
	vector<match> matches;

	// Get the search terms from the command line
	for(int i = 0; i < num_terms; i++)
	{
		search[i] = argv[i + 1];
	}

	// Read in the matrix
	FILE* fp = fopen("matrix.txt", "r");
	if(fp == NULL) { printf("%s\n", "matrix error"); return 1; }
	while(!feof(fp))
	{
		if(fgets(buffer, 80, fp) != NULL)
		{
			if(buffer[0] == '%') { continue; }
			
			sscanf(buffer, "%s%s%s", a, b, c);
				
			if(i++ == 0)
			{
				termByDoc = gsl_matrix_calloc(atoi(a), atoi(b));
				searchTerm = gsl_vector_calloc(atoi(a));
			}
			else
			{
				gsl_matrix_set(termByDoc, atoi(a) - 1, atoi(b) - 1, (double) atoi(c));
			}
		}
	}

	fclose(fp);


	// Read in the term vector and prepare the search vector
	fp = fopen("terms.txt", "r");
	if(fp == NULL) { printf("%s\n", "term error"); return 1; }
	i = 0;
	while(!feof(fp))
	{
		if(fgets(buffer, 80, fp) != NULL)
		{
			sscanf(buffer, "%s", a);

			term.push_back(a);

			for(int j = 0; j < num_terms; j++)
			{
				// If the current term is also a member
				// of the search vector, increment this
				// value in the search vector
				if(strcmp(search[j], a) == 0)
				{
					num_matches++;
					current = gsl_vector_get(searchTerm, i);
					gsl_vector_set(searchTerm, i, ++current);
				}
			}
			i++;
		}
	}

	// Normalize the search vector
	gsl_vector_scale(searchTerm, 1.0 / sqrt(num_matches));

	fclose(fp);

	// Read in the document vector
	fp = fopen("documents.txt", "r");
	if(fp == NULL) { printf("%s\n", "doc error"); return 1; }
	i = 0;
	while(!feof(fp))
	{
		if(fgets(buffer, 80, fp) != NULL)
		{
			sscanf(buffer, "%s", a);

			doc.push_back(a);		
		}
	}

	fclose(fp);

/*
 * /// Computing the SVD is too expensive to be done on the fly.  If the SVD
   /// is to be used, it should be used in a client-daemon environment (as
   /// suggested in the accompanying text), or as a separate process
   /// completely (then read in, as is done with the current matrix)
	// Find the singular value decomposition of the term-by-doc matrix
	// Use the modified Golub-Reinsch algorithm because it is faster
	// for M >> N
	gsl_matrix* X = gsl_matrix_alloc(termByDoc->size2, termByDoc->size2);
	gsl_matrix* V = gsl_matrix_alloc(termByDoc->size2, termByDoc->size2);
	gsl_vector* S = gsl_vector_alloc(termByDoc->size2);
	gsl_vector* work = gsl_vector_alloc(termByDoc->size2);
	gsl_linalg_SV_decomp_mod(termByDoc, X, V, S, work);
*/

	// With the matrix and vectors setup, perform the query
	double result;
	gsl_vector* col = gsl_vector_calloc(termByDoc->size1);
	match temp_match;

	for(int k = 0; k < termByDoc->size2 && matches.size() < MATCH_LIMIT; k++)
	{
		gsl_matrix_get_col(col, termByDoc, k);

		// Normalize the document vector
		gsl_blas_ddot(col, col, &result);
		gsl_vector_scale(col, 1.0 / sqrt(result));

		gsl_blas_ddot(searchTerm, col, &result);

		if(result > THRESHOLD)
		{
			temp_match.doc_index = k;
			temp_match.cosine = result;
			matches.push_back(temp_match);

			// printf("%s\n", k, doc[k].c_str());
		}
	}

	if(matches.size() == 0)
	{
		printf("%s\n", "no matching records");
	}
	else if(matches.size() == 1)
	{
		printf("%s\n", doc[matches[0].doc_index].c_str());
	}
	else
	{
		sort(matches.begin(), matches.end());

		for(int i = matches.size() - 1; i >= 0; i--)
		{
			printf("%s\n", doc[matches[i].doc_index].c_str());
		}
	}

	return 0;
}
